/**
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.component.poll

import org.apache.camel.impl.ServiceSupport
import org.apache.camel.Consumer
import org.apache.camel.Endpoint
import metridoc.utils.StringUtils
import org.apache.camel.ConsumerTemplate
import metridoc.utils.Assert
import org.apache.camel.Processor
import org.apache.camel.Exchange
import org.slf4j.LoggerFactory
import org.apache.camel.component.file.GenericFileEndpoint
import metridoc.polling.OnePollOnlyStrategy
import metridoc.utils.CollectionUtils.Maps

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 4/22/11
 * Time: 10:26 AM
 * To change this template use File | Settings | File Templates.
 */
class PollConsumer extends ServiceSupport implements Consumer{

    def PollEndpoint endpoint
    def ConsumerTemplate consumerTemplate
    def Processor processor
    def static final propertiesToRemove = ["maxMessages", "wait", "usePolling"]
    def static final log = LoggerFactory.getLogger(PollConsumer.class)

    def Endpoint getEndpoint() {
        return endpoint
    }

    @Override
    protected void doStart() {
        def postFix = "cannot be null in PollConsumer when doStart() is called"

        Assert.notNull processor, "the processor ${postFix}"
        Assert.notNull endpoint, "the endpoint ${postFix}"

        processMessages()
    }

    private void processMessages() {
        def consumerTemplate = getConsumerTemplate()
        def maxMessages = endpoint.maxMessages
        def wait = endpoint.wait
        def uri = getEndpoint().getEndpointUri()
        def formattedUri = formatUri(uri)
        def endpointToConsume = resolveEndpoint(formattedUri)

        for (i in (1..maxMessages)) {
            Exchange exchange = consumerTemplate.receive(endpointToConsume, wait)
            if(exchange == null) {
                return
            }

            processor.process exchange
        }

    }

    @Override
    protected void doStop() {
        //Do nothing
    }

    def ConsumerTemplate getConsumerTemplate() {

        Assert.notNull endpoint, "the endpoint cannot be null"

        if(consumerTemplate == null) {
            consumerTemplate = endpoint.camelContext.createConsumerTemplate()
        }

        return consumerTemplate
    }

    private static String formatUri(String uri) {
        def newUri = removePollPrefix(uri)

        propertiesToRemove.each {String name ->
            newUri = removePropertyFromUrl(newUri, name)
        }

        return newUri
    }

    private static String removePollPrefix(String url) {
        def newUrl = url.replace("poll://", StringUtils.EMPTY)

        if(url == newUrl) {
            newUrl = newUrl.replace("poll:", StringUtils.EMPTY)
        }

        return newUrl
    }

    private Endpoint resolveEndpoint(String uri) {
        def camelContext = getEndpoint().getCamelContext()
        def resolvedEndpoint = camelContext.getEndpoint(uri)

        if(resolvedEndpoint instanceof GenericFileEndpoint) {

            resolvedEndpoint.consumerProperties = Maps.createHashMap()
            resolvedEndpoint.consumerProperties["initialDelay"] = 0
            resolvedEndpoint.consumerProperties["delay"] = Long.MAX_VALUE
            resolvedEndpoint.maxMessagesPerPoll = endpoint.maxMessages
        }

        return resolvedEndpoint
    }

    /**
     *
     * @param url the url to adjust
     * @param propertyName the property to remove
     * @return the new url with the property removed
     */
    private static String removePropertyFromUrl(String url, String propertyName) {
        def m = url =~ /(${propertyName}=[^&]+)/
        def String newUrl = url


        if(m.find()) {
            def property = m[0][1]
            newUrl = url.replace(property, StringUtils.EMPTY)
            newUrl = newUrl.replace("?&", "?")
            newUrl = newUrl.replace("&&", "&")
            newUrl = newUrl.replaceFirst("&\$", StringUtils.EMPTY)
            newUrl = newUrl.replaceFirst("\\?\$", StringUtils.EMPTY)
        }

        return newUrl
    }
}
