/**
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package metridoc.dsl

import org.springframework.jdbc.datasource.embedded.EmbeddedDatabaseBuilder
import org.springframework.jdbc.datasource.embedded.EmbeddedDatabaseType
import com.mchange.v2.c3p0.ComboPooledDataSource
import org.slf4j.LoggerFactory
import javax.sql.DataSource
import org.apache.camel.CamelContext
import org.apache.camel.spi.BrowsableEndpoint
import org.apache.camel.ShutdownRunningTask
import org.apache.camel.impl.DefaultCamelContext
import org.apache.camel.ProducerTemplate
import metridoc.registry.MetridocSimpleRegistry

import metridoc.utils.ObjectUtils
import liquibase.integration.spring.SpringLiquibase
import metridoc.utils.StringUtils
import org.springframework.core.io.DefaultResourceLoader
import org.apache.camel.spi.Registry
import metridoc.registry.RegistryWrapper
import org.apache.camel.Endpoint
import javax.naming.OperationNotSupportedException
import metridoc.schema.loader.GroovyClassSchemaResource

/**
 *
 * @author tbarker
 */
@Mixin(PropertyMissing.class)
class JobRunner {

    def CamelContext camelContext
    def ProducerTemplate producerTemplate
    def log = LoggerFactory.getLogger(JobRunner.class)
    def Exception lastException

    def createSchema(args, Closure closure) {
        def schema = new SchemaBuilder(args)
        schema.createColumns closure
        schema.build()
    }

    def liquibase(args) {
        def liquibase = new SpringLiquibase()
        liquibase.dataSource = args.dataSource
        def schema = args.schema

        if(schema instanceof Class) {
            liquibase.resourceLoader = new GroovyClassSchemaResource()
            liquibase.changeLog = "${schema.name}.xml" //adding the xml extension tricks liquibase into using the xml parser
        } else {
            liquibase.changeLog = args.schema
            liquibase.resourceLoader = new DefaultResourceLoader()
        }

        liquibase.contexts = StringUtils.EMPTY
        liquibase.afterPropertiesSet()
    }

    def run(Closure closure) {
        closure.setDelegate(this)
        closure.setDirective(Closure.DELEGATE_ONLY)
        try {
            closure()
            if (lastException) {
                throw lastException
            }
        } catch (Exception ex) {
            lastException = ex
            log.error "exception occurred while running job", ex
        }
    }

    def runRoute(String route, body = ObjectUtils.NULL, headers = [:]) {
        return getProducerTemplate().requestBodyAndHeaders(route, body, headers);
    }

    def runAsyncRoute(String route, body = ObjectUtils.NULL, headers = [:]) {
        return getProducerTemplate().asyncRequestBodyAndHeaders(route, body, headers);
    }

    def runJobs(String... jobs) {
        jobs.each {job ->
            manager.runJob job
        }
    }

    def runRoute(args = null, Closure closure) {
        def routeConfig = new RouteConfig()
        if (args != null) {
            routeConfig = new RouteConfig(args)
        }

        def routeBuilder = new GroovyRouteBuilder(route: closure, jobRunner: this, usePolling: routeConfig.usePolling)
        getCamelContext().addRoutes(routeBuilder)

        def routes = routeBuilder.routeCollection.routes
        def context = getCamelContext()
        def inflight = context.inflightRepository
        def boolean activityDetected = true

        while (activityDetected) {


            routes.each {route ->
                def fromDefinitions = route.inputs
                fromDefinitions.each {from ->
                    def uri = from.uri
                    def endpoint = context.getEndpoint(uri)
                    def size = inflight.size(endpoint)

                    if (size == 0) {
                        activityDetected = false
                    }

                    if (endpoint instanceof BrowsableEndpoint && !activityDetected) {
                        size = endpoint.exchanges.size()
                        if (size == 0) {
                            activityDetected = false
                        }
                    }

                }
            }

            if(activityDetected) {
                Thread.sleep(routeConfig.completionCheckPeriod)
            }
        }
    }

    def CamelContext getCamelContext() {
        if (camelContext == null) {
            def registry = manager.registry
            camelContext = new DefaultCamelContext(registry)
            camelContext.shutdownRunningTask = ShutdownRunningTask.CompleteAllTasks
        }

        if (!camelContext.isStarted()) {
            camelContext.start()
        }

        return camelContext
    }

    def ProducerTemplate getProducerTemplate() {
        if (producerTemplate == null) {
            producerTemplate = getCamelContext().createProducerTemplate()
        }

        return producerTemplate
    }

    def DataSource dataSource(args) {
        return new ComboPooledDataSource(args)
    }

    /**
     * creates an embedded in-memory hsql datasource that is generally used for testing
     */
    def DataSource embeddedDataSource() {
        return new EmbeddedDatabaseBuilder().setType(EmbeddedDatabaseType.H2).build()
    }

    def void loadProperties(String... propertyFiles) {
        propertyFiles.each {
            def testFileName = it + "-test.properties"
            def classPathFile = it + ".properties"
            def systemFile = manager.metridocDirectory + "/" + classPathFile
            def boolean fileFound = false

            if (loadPropertyFile(classPathFile)) {
                fileFound = true
            }

            if (loadPropertyFile(systemFile, true)) {
                fileFound = true
            }

            if (loadPropertyFile(testFileName)) {
                fileFound = true
            }

            if (!fileFound) {
                throw new FileNotFoundException("could not find property file ${it}")
            }
        }
    }

    def boolean loadPropertyFile(String propertyFile, boolean systemFile = false) {

        def url
        if (systemFile) {
            def file = new File(propertyFile)

            if (!file.exists()) {
                return false
            }

            url = file.toURI().toURL()
        } else {
            def classLoader = manager.jobClassLoader
            url = classLoader.getResource(propertyFile)
        }

        def boolean foundFile = false

        if (url != null) {

            def props = new Properties()
            props.load(url.newReader())
            def config = new ConfigSlurper().parse(props)
            config.keySet().each {
                manager.services[it] = config[it]
            }
            foundFile = true
        }

        return foundFile
    }

    def Endpoint getEndpoint(String endpointUri) {
        return getCamelContext().getEndpoint(endpointUri)
    }


}

