/**
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package metridoc.utils;

import java.io.*;
import java.net.URL;
import java.util.Enumeration;
import java.util.List;
import java.util.Properties;
import java.util.zip.GZIPInputStream;

import metridoc.utils.CollectionUtils.Lists;
import org.apache.camel.Exchange;
import org.apache.camel.component.file.GenericFile;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author tbarker
 */
public class IOUtils {

    private static final Logger LOG = LoggerFactory.getLogger(IOUtils.class);
    static final String METRIDOC = ".metridoc";
    static final String PROPERTIES_EXT = ".properties";

    public static final String HOME = System.getProperty("user.home");
    public static final String FILE_SEPARATOR = System.getProperty("file.separator");
    public static final String LINE_SEPARATOR = System.getProperty("line.separator");

    private IOUtils(){}

    /**
     * 
     * @param inputStream
     */
    public static void closeQuietly(InputStream inputStream) {
        if (inputStream != null) {
            try {
                inputStream.close();
            } catch (Exception e) {
                //ignore
                LOG.warn("exception occurred when closing a stream. Since we are closing the stream silently "
                        + "the error will be ignored", e);
            }
        }
    }

    /**
     *
     * @param reader
     */
    public static void closeQuietly(BufferedReader reader) {
        if (reader != null) {
            try {
                reader.close();
            } catch (Exception e) {
                //ignore
                LOG.warn("exception occurred when closing a reader. Since we are closing the stream silently "
                        + "the error will be ignored", e);
            }
        }
    }

    /**
     * returns all streams of the specified resources.  Really useful for grabbing files that have the same name and 
     * location within multiple jars.
     * 
     * @param resource
     * @param classLoader
     * @return
     * @throws IOException
     */
    public static List<InputStream> getResources(String resource, ClassLoader classLoader) throws IOException {
        List<InputStream> result = Lists.createArrayList();
        Enumeration<URL> resources = classLoader.getResources(resource);
        while(resources.hasMoreElements()) {
            URL url = resources.nextElement();
            LOG.debug("found resource {} while searching path {}", url, resource);
            result.add(url.openStream());
        }

        return result;
    }

    /**
     * loads properties from &lt;user.home&gt;/.metridoc/&lt;name&gt;.properties if they are there
     * @param name of the property file
     * @return properties
     */
    public static Properties loadProperties(String name) {
        String filePath = getPropertyFilePath(name);
        File file = new File(filePath);
        return loadProperties(file);
    }

    private static Properties loadProperties(File file) {
        Properties result = new Properties();
        String filePath = file.getPath();

        if (file.exists()) {
            InputStream inputStream = null;
            try {
                inputStream = new FileInputStream(file);
                result.load(inputStream);
            } catch (IOException ex) {
                String message = "could not load properties from file " + filePath;
                throw new RuntimeException(message, ex);
            } finally {
                if (inputStream != null) {
                    IOUtils.closeQuietly(inputStream);
                }
            }
        } else {
            LOG.warn("could not find property file {}", filePath);
        }

        return result;
    }

    /**
     * loads properties from &lt;filePath&gt;/&lt;name&gt;.properties if they are there
     *
     * @param name of the property file
     * @param basePath
     * @return properties
     */
    public static Properties loadProperties(String name, String basePath) {
        String fullPath = getPropertyFilePath(basePath,name);
        File file = new File(fullPath);
        
        return loadProperties(file);
    }

    static String getPropertyFilePath(String name) {
        
        String base = HOME + FILE_SEPARATOR + METRIDOC + FILE_SEPARATOR;

        return getPropertyFilePath(base, name);
    }

    static String getPropertyFilePath(String rootFilePath, String name) {
        String path = rootFilePath;

        if (!path.endsWith(FILE_SEPARATOR)) {
            path += FILE_SEPARATOR;
        }

        path += name;

        if (!name.endsWith(PROPERTIES_EXT)) {
            path += PROPERTIES_EXT;
        }

        return path;
    }

    public static InputStream convertGenericFileToInputStream(Exchange exchange) {
        GenericFile<File> genericFile = exchange.getIn().getBody(GenericFile.class);
        File file = exchange.getContext().getTypeConverter().convertTo(File.class, genericFile.getBody());
        String fileName = genericFile.getFileName();

        try {

            InputStream inputStream;
            if(file != null) {
                inputStream = new FileInputStream(file);
            } else {
                inputStream = exchange.getIn().getBody(InputStream.class);
            }

            if (fileName.endsWith(".gz")) {
                return new GZIPInputStream(inputStream);
            }
            return inputStream;
        } catch (IOException ex) {
            throw new RuntimeException("could not find the file", ex);
        }
    }
}
